-- Test payment
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(71);

set search_path to numerus, auth, public;

select has_table('payment');
select has_pk('payment');
select table_privs_are('payment', 'guest', array []::text[]);
select table_privs_are('payment', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment', 'authenticator', array []::text[]);

select has_column('payment', 'payment_id');
select col_is_pk('payment', 'payment_id');
select col_type_is('payment', 'payment_id', 'integer');
select col_not_null('payment', 'payment_id');
select col_hasnt_default('payment', 'payment_id');

select has_column('payment', 'company_id');
select col_is_fk('payment', 'company_id');
select fk_ok('payment', 'company_id', 'company', 'company_id');
select col_type_is('payment', 'company_id', 'integer');
select col_not_null('payment', 'company_id');
select col_hasnt_default('payment', 'company_id');

select has_column('payment', 'slug');
select col_is_unique('payment', 'slug');
select col_type_is('payment', 'slug', 'uuid');
select col_not_null('payment', 'slug');
select col_has_default('payment', 'slug');
select col_default_is('payment', 'slug', 'gen_random_uuid()');

select has_column('payment', 'description');
select col_type_is('payment', 'description', 'text');
select col_not_null('payment', 'description');
select col_hasnt_default('payment', 'description');

select has_column('payment', 'payment_date');
select col_type_is('payment', 'payment_date', 'date');
select col_not_null('payment', 'payment_date');
select col_has_default('payment', 'payment_date');
select col_default_is('payment', 'payment_date', 'CURRENT_DATE');

select has_column('payment', 'payment_account_id');
select col_is_fk('payment', 'payment_account_id');
select fk_ok('payment', 'payment_account_id', 'payment_account', 'payment_account_id');
select col_type_is('payment', 'payment_account_id', 'integer');
select col_not_null('payment', 'payment_account_id');
select col_hasnt_default('payment', 'payment_account_id');

select has_column('payment', 'amount');
select col_type_is('payment', 'amount', 'integer');
select col_not_null('payment', 'amount');
select col_hasnt_default('payment', 'amount');

select has_column('payment', 'currency_code');
select col_is_fk('payment', 'currency_code');
select fk_ok('payment', 'currency_code', 'currency', 'currency_code');
select col_type_is('payment', 'currency_code', 'text');
select col_not_null('payment', 'currency_code');
select col_hasnt_default('payment', 'currency_code');

select has_column('payment', 'tags');
select col_type_is('payment', 'tags', 'tag_name[]');
select col_not_null('payment', 'tags');
select col_has_default('payment', 'tags');
select col_default_is('payment', 'tags', '{}');

select has_column('payment', 'payment_status');
select col_is_fk('payment', 'payment_status');
select fk_ok('payment', 'payment_status', 'payment_status', 'payment_status');
select col_type_is('payment', 'payment_status', 'text');
select col_not_null('payment', 'payment_status');
select col_has_default('payment', 'payment_status');
select col_default_is('payment', 'payment_status', 'complete');

select has_column('payment', 'created_at');
select col_type_is('payment', 'created_at', 'timestamp with time zone');
select col_not_null('payment', 'created_at');
select col_has_default('payment', 'created_at');
select col_default_is('payment', 'created_at', 'CURRENT_TIMESTAMP');


set client_min_messages to warning;
truncate payment cascade;
truncate payment_account cascade;
truncate company_user cascade;
truncate company cascade;
truncate payment_method cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into payment_account (payment_account_id, company_id, payment_account_type, name)
values (221, 2, 'other', 'Other 2')
     , (441, 4, 'other', 'Other 4')
;

insert into payment (company_id, description, payment_account_id, amount, currency_code)
values (2, 'Payment 20001', 221, 333, 'EUR')
     , (4, 'Payment 40001', 441, 555, 'EUR')
;


prepare payment_data as
select company_id, description
from payment
order by company_id, description;

set role invoicer;
select is_empty('payment_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'payment_data',
	$$ values (2, 'Payment 20001')
	$$,
	'Should only list payments from the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'payment_data',
	$$ values (4, 'Payment 40001')
	$$,
	'Should only list payments from the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'payment_data',
	'42501', 'permission denied for table payment',
	'Should not allow select to guest users'
);
reset role;

select throws_ok(
	$$ insert into payment (company_id, description, payment_account_id, amount, currency_code) values (2, 'Nope', 221, 0, 'EUR') $$,
	'23514', 'new row for relation "payment" violates check constraint "payment_amount_positive"',
	'Should not allow empty payments'
);

select throws_ok(
	$$ insert into payment (company_id, description, payment_account_id, amount, currency_code) values (2, 'Nope', 221, -1, 'EUR') $$,
	'23514', 'new row for relation "payment" violates check constraint "payment_amount_positive"',
	'Should not allow negative payments'
);


select *
from finish();

rollback;

